@@ -2,7 +2,7 @@
 """beanstalkc - A beanstalkd Client Library for Python"""
 
 __license__ = '''
-Copyright (C) 2008 Andreas Bolka
+Copyright (C) 2008, 2009 Andreas Bolka
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -17,8 +17,9 @@ See the License for the specific language governing permissions and
 limitations under the License.
 '''
 
-__version__ = '0.1.1'
+__version__ = '0.2.0'
 
+import logging
 import socket
 
 
@@ -33,28 +34,42 @@ class UnexpectedResponse(BeanstalkcException): pass
 class CommandFailed(BeanstalkcException): pass
 class DeadlineSoon(BeanstalkcException): pass
 
+class SocketError(BeanstalkcException):
+    @staticmethod
+    def wrap(fn, *args, **kwargs):
+        try:
+            return fn(*args, **kwargs)
+        except socket.error, e:
+            raise SocketError(e)
+
 
 class Connection(object):
-    def __init__(self, host=DEFAULT_HOST, port=DEFAULT_PORT, decode_yaml=True):
-        if decode_yaml:
-            self.yaml_load = __import__('yaml').load
-        else:
-            self.yaml_load = lambda x: x
+    def __init__(self, host=DEFAULT_HOST, port=DEFAULT_PORT, parse_yaml=True):
+        if parse_yaml is True:
+            try:
+                parse_yaml = __import__('yaml').load
+            except ImportError:
+                logging.error('Failed to load PyYAML, will not parse YAML')
+                parse_yaml = False
+        self._parse_yaml = parse_yaml or (lambda x: x)
         self.host = host
         self.port = port
         self.connect()
 
     def connect(self):
-        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.socket.connect((self.host, self.port))
-        self.socket_file = self.socket.makefile('rb')
+        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        SocketError.wrap(self._socket.connect, (self.host, self.port))
+        self._socket_file = self._socket.makefile('rb')
 
     def close(self):
-        self.socket.close()
+        try:
+            self._socket.close()
+        except socket.error:
+            pass
 
-    def interact(self, command, expected_ok, expected_err=[]):
-        self.socket.send(command)
-        status, results = self.read_response()
+    def _interact(self, command, expected_ok, expected_err=[]):
+        SocketError.wrap(self._socket.sendall, command)
+        status, results = self._read_response()
         if status in expected_ok:
             return results
         elif status in expected_err:
@@ -62,31 +77,36 @@ class Connection(object):
         else:
             raise UnexpectedResponse(command.split()[0], status, results)
 
-    def read_response(self):
-        response = self.socket_file.readline().split()
+    def _read_response(self):
+        line = SocketError.wrap(self._socket_file.readline)
+        if not line:
+            raise SocketError()
+        response = line.split()
         return response[0], response[1:]
 
-    def read_body(self, size):
-        body = self.socket_file.read(size)
-        self.socket_file.read(2) # trailing crlf
+    def _read_body(self, size):
+        body = SocketError.wrap(self._socket_file.read, size)
+        SocketError.wrap(self._socket_file.read, 2) # trailing crlf
+        if size > 0 and not body:
+            raise SocketError()
         return body
 
-    def interact_value(self, command, expected_ok, expected_err=[]):
-        return self.interact(command, expected_ok, expected_err)[0]
+    def _interact_value(self, command, expected_ok, expected_err=[]):
+        return self._interact(command, expected_ok, expected_err)[0]
 
-    def interact_job(self, command, expected_ok, expected_err, reserved=True):
-        jid, size = self.interact(command, expected_ok, expected_err)
-        body = self.read_body(int(size))
+    def _interact_job(self, command, expected_ok, expected_err, reserved=True):
+        jid, size = self._interact(command, expected_ok, expected_err)
+        body = self._read_body(int(size))
         return Job(self, int(jid), body, reserved)
 
-    def interact_yaml(self, command, expected_ok, expected_err=[]):
-        size, = self.interact(command, expected_ok, expected_err)
-        body = self.read_body(int(size))
-        return self.yaml_load(body)
+    def _interact_yaml(self, command, expected_ok, expected_err=[]):
+        size, = self._interact(command, expected_ok, expected_err)
+        body = self._read_body(int(size))
+        return self._parse_yaml(body)
 
-    def interact_peek(self, command):
+    def _interact_peek(self, command):
         try:
-            return self.interact_job(command, ['FOUND'], ['NOT_FOUND'], False)
+            return self._interact_job(command, ['FOUND'], ['NOT_FOUND'], False)
         except CommandFailed, (_, status, results):
             return None
 
@@ -94,7 +114,7 @@ class Connection(object):
 
     def put(self, body, priority=DEFAULT_PRIORITY, delay=0, ttr=DEFAULT_TTR):
         assert isinstance(body, str)
-        jid = self.interact_value(
+        jid = self._interact_value(
                 'put %d %d %d %d\r\n%s\r\n' %
                     (priority, delay, ttr, len(body), body),
                 ['INSERTED', 'BURIED'])
@@ -106,9 +126,9 @@ class Connection(object):
         else:
             command = 'reserve\r\n'
         try:
-            return self.interact_job(command,
-                                     ['RESERVED'],
-                                     ['DEADLINE_SOON', 'TIMED_OUT'])
+            return self._interact_job(command,
+                                      ['RESERVED'],
+                                      ['DEADLINE_SOON', 'TIMED_OUT'])
         except CommandFailed, (_, status, results):
             if status == 'TIMED_OUT':
                 return None
@@ -116,76 +136,81 @@ class Connection(object):
                 raise DeadlineSoon(results)
 
     def kick(self, bound=1):
-        return int(self.interact_value('kick %d\r\n' % bound, ['KICKED']))
+        return int(self._interact_value('kick %d\r\n' % bound, ['KICKED']))
 
     def peek(self, jid):
-        return self.interact_peek('peek %d\r\n' % jid)
+        return self._interact_peek('peek %d\r\n' % jid)
 
     def peek_ready(self):
-        return self.interact_peek('peek-ready\r\n')
+        return self._interact_peek('peek-ready\r\n')
 
     def peek_delayed(self):
-        return self.interact_peek('peek-delayed\r\n')
+        return self._interact_peek('peek-delayed\r\n')
 
     def peek_buried(self):
-        return self.interact_peek('peek-buried\r\n')
+        return self._interact_peek('peek-buried\r\n')
 
     def tubes(self):
-        return self.interact_yaml('list-tubes\r\n', ['OK'])
+        return self._interact_yaml('list-tubes\r\n', ['OK'])
 
     def using(self):
-        return self.interact_value('list-tube-used\r\n', ['USING'])
+        return self._interact_value('list-tube-used\r\n', ['USING'])
 
     def use(self, name):
-        return self.interact_value('use %s\r\n' % name, ['USING'])
+        return self._interact_value('use %s\r\n' % name, ['USING'])
 
     def watching(self):
-        return self.interact_yaml('list-tubes-watched\r\n', ['OK'])
+        return self._interact_yaml('list-tubes-watched\r\n', ['OK'])
 
     def watch(self, name):
-        return int(self.interact_value('watch %s\r\n' % name, ['WATCHING']))
+        return int(self._interact_value('watch %s\r\n' % name, ['WATCHING']))
 
     def ignore(self, name):
         try:
-            return int(self.interact_value('ignore %s\r\n' % name,
-                                           ['WATCHING'],
-                                           ['NOT_IGNORED']))
+            return int(self._interact_value('ignore %s\r\n' % name,
+                                            ['WATCHING'],
+                                            ['NOT_IGNORED']))
         except CommandFailed:
             return 1
 
     def stats(self):
-        return self.interact_yaml('stats\r\n', ['OK'])
+        return self._interact_yaml('stats\r\n', ['OK'])
 
     def stats_tube(self, name):
-        return self.interact_yaml('stats-tube %s\r\n' % name,
+        return self._interact_yaml('stats-tube %s\r\n' % name,
                                   ['OK'],
                                   ['NOT_FOUND'])
 
+    def pause_tube(self, name, delay):
+        self._interact('pause-tube %s %d\r\n' %(name, delay),
+                       ['PAUSED'],
+                       ['NOT_FOUND'])
+
     # -- job interactors --
 
     def delete(self, jid):
-        self.interact('delete %d\r\n' % jid, ['DELETED'])
+        self._interact('delete %d\r\n' % jid, ['DELETED'], ['NOT_FOUND'])
 
     def release(self, jid, priority=None, delay=0):
-        self.interact('release %d %d %d\r\n' % (jid, priority, delay),
-                      ['RELEASED', 'BURIED'],
-                      ['NOT_FOUND'])
+        self._interact('release %d %d %d\r\n' % (jid, priority, delay),
+                       ['RELEASED', 'BURIED'],
+                       ['NOT_FOUND'])
 
     def bury(self, jid, priority=None):
-        self.interact('bury %d %d\r\n' % (jid, priority),
-                      ['BURIED'],
-                      ['NOT_FOUND'])
+        self._interact('bury %d %d\r\n' % (jid, priority),
+                       ['BURIED'],
+                       ['NOT_FOUND'])
 
     def touch(self, jid):
-        self.interact('touch %d\r\n' % jid, ['TOUCHED'], ['NOT_FOUND'])
+        self._interact('touch %d\r\n' % jid, ['TOUCHED'], ['NOT_FOUND'])
 
     def stats_job(self, jid):
-        return self.interact_yaml('stats-job %d\r\n' % jid,
-                                  ['OK'],
-                                  ['NOT_FOUND'])
+        return self._interact_yaml('stats-job %d\r\n' % jid,
+                                   ['OK'],
+                                   ['NOT_FOUND'])
 
 
-class Job:
+class Job(object):
     def __init__(self, conn, jid, body, reserved=True):
         self.conn = conn
         self.jid = jid
@@ -217,7 +242,7 @@ class Job:
 
 
 if __name__ == '__main__':
-    import doctest, os, signal, time
+    import doctest, os, signal
     try:
         pid = os.spawnlp(os.P_NOWAIT,
                          'beanstalkd',
